<?php

namespace Utils;

use Config\Config;

/**
 * 工具类
 * Class ApiUtil
 */
class ApiUtil
{

    /**
     * 获取应用凭证
     *
     * @return mixed
     */
    public static function getAppToken()
    {
        $sendData = [
            "app_key" => Config::APP_KEY,
            "app_secret" => Config::APP_SECRET,
            "grant_type" => "client_credentials",
            "scope" => "base",
            "ocode" => Config::SCHOOL_CODE
        ];
        $sendData['signature'] = SignUtil::calSign($sendData);
        return json_decode(self::httpCurlS(Config::APP_ACCESS_TOKEN, $sendData), true);
    }

    /**
     * 电子码解码
     *
     * @param $data
     * @return mixed
     */
    public static function decode($data)
    {
        if (!$data) {
            return [];
        }
        $sendData = [
            "app_key" => Config::APP_KEY,
            "timestamp" => time(),
            "nonce" => EncryptUtil::random(32),//随机32位,
            "school_code" => Config::SCHOOL_CODE,
            "auth_code" => $data["code"],
            "scene" => $data["scene"],
            "device_no" => $data["device_no"],
            "location" => $data["location"]
        ];
        $sendData['signature'] = SignUtil::calSign($sendData);
        return json_decode(self::httpCurlS(Config::CAMPUS_CODE, $sendData), true);
    }

    /**
     * 电子码回调
     *
     * @param $data
     * @return mixed
     */
    public static function codeNotify($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        $sendData = [
            "app_key" => Config::APP_KEY,
            "card_number" => $data["card_number"],
            "event_id" => $data["event_id"],
            "nonce" => EncryptUtil::random(32),
            "school_code" => Config::SCHOOL_CODE,
            "timestamp" => time(),
            'params' => $data['params'],
        ];

        $sendData['signature'] = SignUtil::calSign($sendData);
        return json_decode(self::httpCurlS(Config::CAMPUS_NOTIFY, $sendData), true);
    }

    /**
     * 获取access_token
     *
     * @param $data
     * @return mixed
     */
    public static function getAccessTokenAndUser($data)
    {
        if ($data['wxcode']) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }
        $sendData = [
            "wxcode" => $data["wxcode"],
            "app_key" => Config::APP_KEY,
            "app_secret" => Config::APP_SECRET,
            "grant_type" => "authorization_code",
            "redirect_uri" => Config::REDIRECT_URL
        ];
        $tokenInfo = json_decode(self::httpCurlS(Config::GET_ACCESS_TOKEN, $sendData), true);
        $userInfo = [];
        if ($tokenInfo["access_token"]) {
            $userInfo = json_decode(self::httpCurlS(Config::GET_LOGIN_INFO_URL, ["access_token" => $tokenInfo["access_token"]]), true);
        }
        $result['tokenInfo'] = $tokenInfo;
        $result['userInfo'] = $userInfo;
        return $result;
    }


    /**
     * 获取当前登录者的信息
     *
     * @param $data
     * @return mixed
     */
    public static function getLoginUserInfo($data)
    {
        $sendData = [
            "access_token" => $data["access_token"],
        ];
        return json_decode(self::httpCurlS(Config::GET_LOGIN_INFO_URL, $sendData), true);
    }

    /**
     * 获取机具配置
     *
     * @return mixed
     */
    public static function getMachineConfig()
    {
        return json_decode(self::httpCurlS(Config::CONFIG_URL, [], 'POST', self::getHeader()), true);
    }

    /**
     * 机具心跳上报
     *
     * @param $can_pay
     * @return mixed
     */
    public static function machineHeartbeat($can_pay)
    {
        if (!$can_pay) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        $sendData = [
            "timestamp" => time(),
            "can_pay" => $can_pay,
        ];
        return json_decode(self::httpCurlS(Config::HEART_URL, $sendData, 'POST', self::getHeader()), true);
    }


    /**
     * 机具支付代扣
     *
     * @param array $data
     * @return mixed
     */
    public static function pay($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        $sendData = [
            'order_id' => $data["order_id"],
            'order_time' => $data["order_time"],
            'amount' => $data["amount"],
            'scene' => $data["scene"],
            'auth_code' => $data["auth_code"],
            'offline' => $data["offline"],
            'md5' => $data["md5"],
        ];
        return json_decode(self::httpCurlS(Config::PAPPAY_URL, $sendData, 'POST', self::getHeader()), true);
    }

    /**
     * 机具支付退款
     *
     * @param array $data
     * @return mixed
     */
    public static function refund($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }
        return json_decode(self::httpCurlS(Config::REFUND_URL, $data, 'POST', self::getHeader()), true);
    }

    /**
     * 机具支付退款查询
     *
     * @param array $data
     * @return mixed
     */
    public static function refundQuery($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        $sendData = [
            'refund_id' => $data["refund_id"],
        ];
        return json_decode(self::httpCurlS(Config::REFUND_QUERY_URL, $sendData, 'POST', self::getHeader()), true);
    }

    /**
     * 机具注册
     *
     * @return mixed
     */
    public static function machineRegister()
    {
        $sendData = [
            'org_id' => Config::SCHOOL_CODE,
        ];
        return json_decode(self::httpCurlS(Config::REGISTER_URL, $sendData, 'POST', self::getHeader()), true);
    }


    /**
     * 机具今日数据
     *
     * @return mixed
     */
    public static function statToday()
    {
        return json_decode(self::httpCurlS(Config::REGISTER_URL, [], 'POST', self::getHeader()), true);
    }

    /**
     * 生成机具相关的头部验证信息
     *
     * @return string[]
     */
    public static function getHeader()
    {
        #生成随机字符
        $str = EncryptUtil::random(10) . ":" . time() . ":" . Config::MACHINE_VERSION;
        $key = Config::MACHINE_FINGER_PRINT;
        $iv = substr($key, 0, 16);
        $headerStr = EncryptUtil::encrypt($str, $key, $iv);
        return ["authorization: WxToken " . Config::MACHINE_ID . ":" . $headerStr];

    }


    /**
     * 获取成员身份类型
     *
     * @param $data
     * @return mixed
     */
    public static function getMemberIdentityType($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        return json_decode(self::httpCurlS(Config::IDENTITY_TYPE, $data), true);
    }


    /**
     * 获取单个组织成员
     *
     * @param $data
     * @return mixed
     */
    public static function getSingleOrgMember($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        $sendData["access_token"] = $data["access_token"];
        $sendData['org_id'] = $data['org_id'];
        $sendData['fetch_child'] = $data["fetch_child"];
        $sendData['page'] = $data["page"];
        $sendData['page_size'] = $data["page_size"];
        return json_decode(self::httpCurlS(Config::SINGLE_ORG_MEMBER, $sendData), true);
    }


    /**
     * 搜索成员
     *
     * @param $data
     * @return mixed
     */
    public static function searchMember($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        return json_decode(self::httpCurlS(Config::SEARCH_MEMBER, $data), true);
    }


    /**
     * 获取组织架构
     *
     * @param $data
     * @return mixed
     */
    public static function getOrg($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        return json_decode(self::httpCurlS(Config::ORG, $data), true);
    }


    /**
     * 获取组织架构类型
     *
     * @param $data
     * @return mixed
     */
    public static function getOrgType($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        return json_decode(self::httpCurlS(Config::ORG_TYPE, $data), true);
    }


    /**
     * 获取登录者权限
     *
     * @param $data
     * @return mixed
     */
    public static function getMemberPermission($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        return json_decode(self::httpCurlS(Config::GET_USER_PERMISSION, $data), true);
    }


    /**
     * 获取所属组的权限
     *
     * @param $data
     * @return mixed
     */
    public static function getGroupPermission($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        return json_decode(self::httpCurlS(Config::GET_GROUP_PERMISSION, $data), true);
    }


    /**
     * 获取权限详情
     *
     * @param $data
     * @return mixed
     */
    public static function getPermissionDetail($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        return json_decode(self::httpCurlS(Config::PERMISSION_DETAIL, $data), true);
    }

    /**
     * 获取角色
     *
     * @param $data
     * @return mixed
     */
    public static function getRole($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        return json_decode(self::httpCurlS(Config::GET_ROLE, $data), true);
    }

    /**
     * 获取角色组
     *
     * @param $data
     * @return mixed
     */
    public static function getRoleGroup($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        return json_decode(self::httpCurlS(Config::GET_ROLE_GROUP, $data), true);
    }


    /**
     * 获取角主体信息
     *
     * @param $data
     * @return mixed
     */
    public static function getMainInfo($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        return json_decode(self::httpCurlS(Config::SCHOOL_INFO, $data), true);
    }


    /**
     * 查询通知能力
     *
     * @return mixed
     */
    public static function getNoticeAbility()
    {

        $sendData['school_code'] = Config::SCHOOL_CODE;
        $sendData['app_key'] = Config::APP_KEY;
        $sendData['timestamp'] = time();
        $sendData['nonce'] = EncryptUtil::random(32);
        $sendData['signature'] = SignUtil::calSign($sendData);
        return json_decode(self::httpCurlS(Config::GET_NOTICE_ABILITY, $sendData), true);
    }


    /**
     * 通知数据明细
     *
     * @param $mark
     * @return mixed
     */
    public static function noticeDataDetail($mark)
    {
        if (!$mark) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        $sendData['mark'] = $mark;
        $sendData['app_key'] = Config::APP_KEY;
        $sendData['timestamp'] = time();
        $sendData['nonce'] = EncryptUtil::random(32);
        $sendData['signature'] = SignUtil::calSign($sendData);
        return json_decode(self::httpCurlS(Config::GET_NOTICE_DETAIL, $sendData), true);
    }


    /**
     * 发送通知
     *
     * @param $data
     * @return mixed
     */
    public static function sendNotice($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        $sendData['school_code'] = Config::SCHOOL_CODE;
        $sendData['cards'] = $data["cards"];
        $sendData['title'] = $data["title"];
        $sendData['content'] = $data["content"];
        $sendData['sender'] = $data["sender"];
        $sendData['timestamp'] = time();
        $sendData['nonce'] = EncryptUtil::random(32);
        $sendData['digest'] = $data["digest"];
        $sendData['customs'] = $data["customs"];
        $sendData['signature'] = SignUtil::calSign($sendData);

        return json_decode(self::httpCurlS(Config::SEND_NOTICE, $sendData), true);
    }


    /**
     * 用户阅读情况
     *
     * @param $data
     * @return mixed
     */
    public static function readInfo($data)
    {
        if (!$data) {
            return ["errorCode" => 9527, "errorMsg" => "参数缺省"];
        }

        $sendData['card_number'] = $data["card_number"];
        $sendData['mark'] = $data["mark"];
        $sendData['app_key'] = Config::APP_KEY;
        $sendData['timestamp'] = time();
        $sendData['nonce'] = EncryptUtil::random(32);
        $sendData['signature'] = SignUtil::calSign($sendData);
        return json_decode(self::httpCurlS(Config::USER_READ, $sendData), true);
    }


    /**
     * 发送post/get请求
     * @param $url //请求地址
     * @param $data //body中参数
     * @param $request_type //请求的接口
     * @param $header //请求头
     * @return mixed
     */
    public static function httpCurlS($url, $data, $request_type = "POST", $header = [])
    {
        $curl_info = array(
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 3,
            CURLINFO_HEADER_OUT => true,#可查看头部设置的参数
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => $request_type,
            CURLOPT_SSL_VERIFYPEER => false,//跳过SSL验证测试环境实际开发开启
            CURLOPT_HTTPHEADER => $header ?: array("Content-Type:application/json"),
        );
        if ($request_type === "POST") {
            $curl_info[CURLOPT_POSTFIELDS] = $data ? json_encode($data) : '';
        }

        $curl = curl_init();
        curl_setopt_array(
            $curl,
            $curl_info
        );
        $response = curl_exec($curl);
        curl_close($curl);
        return $response;
    }
}
